<?php

/**
 * 	CHERRY CHECKOUT - PHP SDK
 * 	CherryCheckout\Api
 *
 * 	author:			Alex Potot
 * 	last update:	2017-08-08
 * 	copyright:		Cherry Checkout SA
 * 	website:		https://www.cherrycheckout.com
 * 	support:		support@cherrycheckout.com
 * 	doc:			https://developers.cherrycheckout.com/en/sdk/php
 */

namespace CherryCheckout;

use CherryCheckout\Me;
use CherryCheckout\Auth;
use CherryCheckout\Order;

use CherryCheckout\Exceptions\CustomerEmailMissingException;
use CherryCheckout\Exceptions\CustomerNotSetException;
use CherryCheckout\Exceptions\NoCharityAvailableException;
use CherryCheckout\Exceptions\OrderNotFoundException;
use CherryCheckout\Exceptions\TrackIdAlreadyExistsException;
use CherryCheckout\Exceptions\TrackIdMissingException;

class Api
{
	private $apiEndpoint;
	private $me;
	private $publicKey;
	private $privateKey;

	/**
	 * CherryCheckoutApi constructor.
	 * @param string $apiEndpoint
	 * 	Url of the Cherry Checkout api endpoint ( https://api.cherrycheckout.com )
	 * @param string $publicKey
	 * 	Your api public key, provided by Cherry Checkout, available in your dashboard ( https://dashboard.cherrycheckout.com )
	 * @param string $privateKey
	 * 	Your api private key, provided by Cherry Checkout, available in your dashboard ( https://dashboard.cherrycheckout.com )
	 */
	public function __construct($apiEndpoint, $publicKey, $privateKey)
	{
		$this->apiEndpoint = $apiEndpoint;
		$this->publicKey = $publicKey;
		$this->privateKey = $privateKey;
	}

	/**
	 * Get me. An object with some data about your app.
	 * @param bool $forceFromApi
	 * @return Me
	 * @throws \Exception
	 */
	public function getMe($forceFromApi = false)
	{
		if ($this->me && !$forceFromApi)
		{
			return $this->me;
		}

		try
		{
			$me = new Me($this->_get('/app/me'));
			$this->me = $me;

			return $this->me;
		}
		catch (\Exception $ex)
		{
			throw $ex;
		}
	}

	/**
	 * Update me. Save data about your app.
	 * @param Me $me
	 * @throws \Exception
	 */
	public function updateMe(Me $me)
	{
		try {
			$this->_post('PUT', '/app/me', $me);
		}
		catch (\Exception $ex) {
			throw $ex;
		}
	}

	/**
	 * Create a new order.
	 * @param Order $order
	 * @throws \Exception
	 */
	public function createOrder(Order $order)
	{
		try {
			$this->_post('POST', '/orders', $order);
		}
		catch (\Exception $ex) {
			switch($ex->getMessage()) {
				case '3001_track_id_already_exists':
					throw new TrackIdAlreadyExistsException('Track ID already exists', 3001);
					break;
				case '3004_no_charity_available':
					throw new NoCharityAvailableException('No charity available', 3004);
					break;
				case '3006_customer_not_set':
					throw new CustomerNotSetException('Customer not set', 3006);
					break;
				case '3010_customer_email_is_missing':
					throw new CustomerEmailMissingException('Customer e-mail is missing', 3010);
					break;
				default:
					throw $ex;
					break;
			}
		}
	}

	/**
	 * Update an existing order.
	 * @param Order $order
	 * @throws \Exception
	 */
	public function updateOrder(Order $order)
	{

		// Throw an error if track id is missing
		if (!isset($order->trackId) || !$order->trackId) {
			throw new TrackIdMissingException('You must specify the track id of the order', 3100);
		}

		try {
			$this->_post('PUT', '/orders/'.$order->trackId, $order);
		}
		catch (\Exception $ex) {
			switch($ex->getMessage()) {
				case '3101_order_not_found':
					throw new OrderNotFoundException('No order found for this track id: ' . $order->trackId, 3101);
					break;
				default:
					throw $ex;
					break;
			}
		}
	}

	/**
	 * Get required headers to make an api call.
	 * @return array
	 */
	private function _getRequestHeaders()
	{
		$timestamp = time();

		// Crypt the (publicKey + timestamp) with the privateKey, in a sha256 algo
		$myKeyEncrypted = Auth::hashKeys($this->publicKey, $this->privateKey, $timestamp);

		return array(
			'Content-type: application/json',
			'authorization: '.$this->publicKey.':'.$myKeyEncrypted,
			'timestamp: '.$timestamp
		);
	}

	/**
	 * GET a request from the api.
	 * @param string $route
	 * 	The route you want to call
	 * @return mixed
	 * @throws \Exception
	 */
	private function _get($route)
	{
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $this->apiEndpoint . $route);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $this->_getRequestHeaders());

		$response = curl_exec($ch);

		$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		// If httpCode is not 200, return an error
		if ($httpCode != 200) {
			if ($response) {
				throw new \Exception($response);
			}
			else if ($httpCode == 404) {
				throw new \Exception('No route found for ' . $route);
			}
			else {
				throw new \Exception('No response from API');
			}
		}

		return json_decode($response);
	}

	/**
	 * POST a request to the api.
	 * @param string $method
	 * 	The method you want to use (POST or PUT)
	 * @param string $route
	 * 	The route you want to call
	 * @param $params
	 * 	The object you want to post/put
	 * @return mixed
	 * @throws \Exception
	 */
	private function _post($method, $route, $params)
	{
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $this->apiEndpoint . $route);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $this->_getRequestHeaders());
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));

		$response = curl_exec($ch);

		$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		// If httpCode is not 200, return an error
		if ($httpCode != 200) {
			if ($response) {
				throw new \Exception($response);
			}
			else {
				throw new \Exception('No response from API');
			}
		}

		return json_decode($response);
	}
}